;;;
;;; Copyright (C) 2010 Frederic Vecoven
;;;
;;; This file is part of NixieClock
;;;
;;; NixieClock is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 3 of the License, or
;;; (at your option) any later version.
;;;
;;; NixieClock is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program. If not, see <http://www.gnu.org/licenses/>.
;;;

;
; Button routines
;
	include hv.inc
	include program.inc
	
	; variables
	extern	but0_tick, but1_tick, but0_ignore, but1_ignore
	extern	BFlag	
	
	
button_code CODE
			
#define LONG	.64
#define BOTH	.128
#define IGNORE	.6

;
; handle_buttons is called from the interrupt routine of timer1
; (every 1/64 sec). The routine detects which button has been pressed,
; and how long. It sets button flags accordingly.
;
handle_buttons
	GLOBAL	handle_buttons
				; button0
button0
	btfsc	BUTTON0_PIN	; is it pressed ?
	bra	b0_released	; no.. handle button0 release
b0_pressed
	infsnz	but0_tick,f	; yes... increment but0_tick
	decf	but0_tick,f	; (and don't wrap)
	movlw	LONG		; button hold for more than LONG ticks ?
	cpfseq	but0_tick
	bra	button1		; no.
	btfss	BUTTON1_PIN	; button 1 is also pressed ?
	bra	button1		; yes. ignore long button 0 for now
	bsf	BFlag,B0_LONG
	movlw	IGNORE
	movwf	but0_ignore
	bra	button1
b0_released
	movf	but0_tick,w	; if but0_tick == 0
	bz	button1		; then done.
	tstfsz	but0_ignore	; check ignore_release
	bra	b0_rel_done
	bsf	BFlag,B0_SHORT
b0_rel_done
	clrf	but0_tick
	;bra	button1	
	
button1
	btfsc	BUTTON1_PIN	; is it pressed ?
	bra	b1_released	; no.. handle button1 release
b1_pressed
	infsnz	but1_tick,f	; yes... increment but1_tick
	decf	but1_tick,f	; (and don't wrap)
	movlw	LONG
	cpfseq	but1_tick
	bra	button2
	btfss	BUTTON0_PIN
	bra	button2
	bsf	BFlag,B1_LONG
	movlw	IGNORE
	movwf	but1_ignore
	bra	button2		; done.
b1_released
	movf	but1_tick,w	; if but1_tick == 0
	bz	button2		; then done.
	tstfsz	but1_ignore	; check ignore_release
	bra	b1_rel_done
	bsf	BFlag,B1_SHORT
b1_rel_done
	clrf	but1_tick
	;bra	button2
	
button2
	movlw	BOTH
	cpfsgt	but0_tick	; if but0_tick > 64
	bra	b2_00
	cpfsgt	but1_tick	; && but1_tick > 64
	bra	b2_00
	bsf	BFlag,B_BOTH	; then both pressed long
	movlw	IGNORE
	movwf	but0_ignore	; set ignore_release for 6 ticks
	movwf	but1_ignore
b2_00
	btfss	BUTTON0_PIN	; if button0 is not pressed
	bra	b2_01
	tstfsz	but0_ignore	; then decrease but0_ignore (if not 0)
	decf	but0_ignore,f
b2_01	
	btfss	BUTTON1_PIN	; if button1 is not pressed
	return
	tstfsz	but1_ignore	; then decrease but1_ignore (if not 0)
	decf	but1_ignore,f
	return
	
	

	END
	