;;;
;;; Copyright (C) 2010 Frederic Vecoven
;;;
;;; This file is part of NixieClock
;;;
;;; NixieClock is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 3 of the License, or
;;; (at your option) any later version.
;;;
;;; NixieClock is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program. If not, see <http://www.gnu.org/licenses/>.
;;;

;
; "Menu" and editor routines
;
	include hv.inc
	include program.inc

	; variables
	extern	BFlag, GFlag, PFlag, time, date, tick, max_bright
	extern	auto_values, auto_brightness, auto_fading
	extern	ed_values, ed_mode, ed_tube, ed_max, ed_tmp
#if SUPPORT_DP == 1
	extern	auto_dp
#endif
	
	; functions
	extern	init_ptrs, get_days, get_days_bcd, get_dow
	extern	jump_table, power_off, hours24_to_12
	extern	inc_max_bright

	
	
; editor modes
#define ED_NONE		0
#define ED_TIME		1
#define ED_DATE		2
#define ED_YEAR		3

	
editor_code CODE

;
; check buttons : call from the main loop
;
check_buttons
	GLOBAL	check_buttons
	
	movf	BFlag,w
	btfsc	STATUS,Z		; any button pressed ?
	return				; no => return
	
	movf	ed_mode,w		; switch (ed_mode)
	call	jump_table
	goto	editor_none		; 0 => none
	goto	editor_time		; 1 => time
	goto	editor_date		; 2 => date
	goto	editor_year		; 3 => year

; 
; Button is pressed, and we are not in editor mode
;
editor_none
	btfsc	BFlag,B0_LONG		; set time
	call	editor_set_time
	btfsc	BFlag,B1_LONG		; power off
	call	to_off
	btfsc	BFlag,B1_SHORT		; cycle maximum brightness
	call	inc_max_bright
	clrf	BFlag	
	return
	
	
; force a power off
to_off
	call	power_off
	bsf	PFlag,POWER_USER_OFF
	return
	
;
; Button is pressed while being in "set time" mode
;
editor_time
	btfsc	BFlag,B0_SHORT		; next tube
	call	edt_next_tube
	btfsc	BFlag,B1_SHORT		; inc digit
	call	edt_next_digit
	btfsc	BFlag,B0_LONG		; set time !
	call	editor_time_set_now
	btfsc	BFlag,B1_LONG		; abort
	call	editor_exit
	clrf	BFlag
	return
edt_next_tube
	incf	ed_tube,f		; tube++
	movlw	.4			; last one ?
	cpfseq	ed_tube
	bra	editor_refresh_4	; no, update
	clrf	ed_tube			; yes, go back to tube 0
	bra	editor_refresh_4
edt_next_digit
	movf	ed_tube,w		; find max allowable value for digit 
	call	time_digit_max		; being edited
	movwf	ed_max
	lfsr	FSR1,ed_values		; compute pointer to current value
	movf	ed_tube,w
	addwf	FSR1L,f
	incf	INDF1,w			; W = digit + 1
	cpfslt	ed_max			; W < max ?
	bra	end_00
	clrf	INDF1			; no, set digit = 0
	call	fix_hour
	bra	editor_refresh_4
end_00
	incf	INDF1,f			; yes, digit++
	call	fix_hour
	bra	editor_refresh_4
fix_hour
	movlw	.2
	cpfseq	ed_values		; tube 1 == '2' ?
	return				; no.
	movlw	.3
	cpfsgt	ed_values+1		; yes. tube 2 > '3' ?
	return
	clrf	ed_values+1		; yes, set it to '0'.
	return


; 
; Time set now !
;
editor_time_set_now
	bcf	INTCON,GIEL
	bcf	PIR1,TMR1IF		; clear eventual pending interrupt
	movlw	0xFE			; reload timer 1
	movwf	TMR1H
	clrf	TMR1L
	clrf	tick
	clrf	t_sec			; second = 0
	clrf	t2_sec
	incf	t2_sec,f
	swapf	ed_values,w		; build hours
	iorwf	ed_values+1,w
	movwf	t_hour
	movwf	t2_hour
	swapf	ed_values+2,w		; build minutes
	iorwf	ed_values+3,w
	movwf	t_min
	movwf	t2_min
	bsf	INTCON,GIEL
	bra	editor_set_year		; propose to set the year

; 
; exit editor : rebuild the ptrs, and update the tubes.
;
editor_exit
	call	init_ptrs
	bsf	GFlag,TUBE_UPDATE
	bcf	GFlag,MANUAL_UPDATE
	clrf	ed_mode
	movff	max_bright+1,max_bright	; restore max_bright
	return


; maximum value for a digit when setting HH:MM
; [0-2] [0-3 (if first digit == 2) or 0-9] [0-5] [0-9]
time_digit_max
	movf	ed_tube,w
	movwf	ed_tmp
	btfsc	STATUS,Z
	retlw	.2			; tube 0 : H. ..  [0-2]
	decfsz	ed_tmp,f
	bra	tdm_00
	movf	ed_values,w
	decf	WREG,w
	decf	WREG,w
	btfsc	STATUS,Z
	retlw	.3			; tube 1 : 2H ..  [0-3]
	retlw	.9			; tube 1 : .H ..  [0-9]
tdm_00
	decfsz	ed_tmp,f
	retlw	.9			; tube 4 : .. .M  [0-9]
	retlw	.5			; tube 3 : .. M.  [0-5]
	
	
	
;
; Put the clock in "set time" mode
;
editor_set_time
	bsf	GFlag,MANUAL_UPDATE	; prevent auto-update
	movlw	ED_TIME			; put the editor in "set time" mode
	movwf	ed_mode
	movff	max_bright,max_bright+1	; save max_bright
	movlw	0x0F
	movwf	max_bright		; and make max_bright 0xF
	clrf	ed_tube			; tube being edited = 0
	swapf	t_hour,w		; copy current time HH:MM to editor 
	andlw	0x0F
	movwf	ed_values
	movf	t_hour,w
	andlw	0x0F
	movwf	ed_values+1
	swapf	t_min,w
	andlw	0x0F
	movwf	ed_values+2
	movf	t_min,w
	andlw	0x0F
	movwf	ed_values+3
	call	editor_refresh_4
	return


;
; Put the clock in "set year" mode
;
editor_set_year
	bsf	GFlag,MANUAL_UPDATE
	movlw	ED_YEAR
	movwf	ed_mode
	movlw	0x2
	movwf	ed_values
	clrf	ed_values+1		; 2 0 x x
	swapf	t_year,w
	andlw	0x0F
	movwf	ed_values+2
	movf	t_year,w
	andlw	0x0F
	movwf	ed_values+3	
	movlw	.2			; first two digit (20) not editable
	movwf	ed_tube
	call	editor_refresh_4
	return
	
	
; button is pressed in "set year" mode
editor_year
	btfsc	BFlag,B0_SHORT		; next tube
	call	edy_next_tube
	btfsc	BFlag,B1_SHORT		; inc digit
	call	edy_next_digit
	btfsc	BFlag,B0_LONG		; set year
	call	editor_year_set_now
	btfsc	BFlag,B1_LONG		; abort
	call	editor_exit
	clrf	BFlag
	return
edy_next_tube
	incf	ed_tube,f
	movlw	.4			; last one ?
	cpfseq	ed_tube
	bra	editor_refresh_4	; no, update
	movlw	.2			; yes, back to third tube
	movwf	ed_tube
	bra	editor_refresh_4
edy_next_digit
	lfsr	FSR1,ed_values		; compute pointer to current value
	movf	ed_tube,w
	addwf	FSR1L,f
	incf	INDF1,w			; increment digit and wrap if needed
	daw
	andlw	0x0F
	movwf	INDF1
	bra	editor_refresh_4
editor_year_set_now
	swapf	ed_values+2,w
	iorwf	ed_values+3,f
	movff	ed_values+3,ed_values+2	; save new year in ed_values+2
	bra	editor_set_date	
	
	
; set the date. Format is MM.DD (more friendly to know the MM first)
editor_set_date
	bsf	GFlag,MANUAL_UPDATE
	movlw	ED_DATE
	movwf	ed_mode
	clrf	ed_tube
	movff	t_month,ed_values	; MM first
	movff	t_day,ed_values+1	; DD second
	call	fix_date
	call	editor_refresh_2
	return
	
	
; button is pressed in "set date" mode
editor_date
	btfsc	BFlag,B0_SHORT		; next pair
	call	edd_next_pair
	btfsc	BFlag,B1_SHORT		; inc value
	call	edd_next_digit
	btfsc	BFlag,B0_LONG		; set date !
	call	edd_date_set_now
	btfsc	BFlag,B1_LONG		; abort
	call	editor_exit
	clrf	BFlag
	return
edd_next_pair
	incf	ed_tube,f
	movlw	.2
	cpfslt	ed_tube
	clrf	ed_tube
	bra	editor_refresh_2
edd_next_digit
	movf	ed_tube,w
	call	date_digit_max
	movwf	ed_max
	lfsr	FSR1,ed_values
	movf	ed_tube,w
	addwf	FSR1L,f
	incf	INDF1,w
	daw
	cpfslt	ed_max
	bra	edd_00
	clrf	INDF1
	call	fix_date
	bra	editor_refresh_2
edd_00	incf	INDF1,w
	daw
	movwf	INDF1
	call	fix_date
	bra	editor_refresh_2
	
fix_date
	tstfsz	ed_values		; if MM == 0
	bra	fd_00
	incf	ed_values,f		; then MM = 1
fd_00	tstfsz	ed_values+1		; if DD == 0
	bra	fd_01
	incf	ed_values+1,f		; then DD = 1
fd_01	
	movff	ed_values+2,date+2
	movff	ed_values,date+1
	call	get_days_bcd		; get max number of days for edited month
	movwf	ed_tmp
	cpfslt	ed_values+1		; if edited days is bigger,
	movwf	ed_values+1		; then make it maximum allowable value.
	return
	

edd_date_set_now
	movf	ed_values,w		; set MM
	movwf	t_month
	movwf	t2_month
	movf	ed_values+1,w		; set DD
	movwf	t_day
	movwf	t2_day
	movf	ed_values+2,w		; set YY
	movwf	t_year
	movwf	t2_year
	call	set_dow
	bra	editor_exit

; date/year has been set. Recompute Day Of Week.
set_dow
	GLOBAL	set_dow
	movff	t_day,date		; copy DD
	movff	t_month,date+1		;      MM
	movff	t_year,date+2		;      YY
	call	get_dow
	movwf	t_dow
	movwf	t2_dow
	return


; maximum value for a pair of tube (MM or DD)
date_digit_max
	movf	ed_tube,w		; if ed_tube == 0
	btfsc	STATUS,Z
	retlw	0x12			; then return 0x12
	movff	ed_values+2,date+2	; else copy YY and MM into 'date'
	movff	ed_values,date+1
	call	get_days_bcd		; and return max days for this month
	return

	

;
; editor refresh : display values being edited, full brightness, no fading
; and update fading for tube being currently edited.
;
editor_refresh_4
	call	editor_display_normal	; full brightness, no fading
	call	editor_display_4_values	; display editor values
	lfsr	FSR1,auto_brightness
	movf	ed_tube,w
	addwf	FSR1L,f
#if NUM_TUBES == 6
	incf	FSR1L,f
#endif
	movlw	(CYCLE_BRIGHT | EVERY_1_32_S)
	movwf	INDF1
	return

; editor refresh for pairs. same as above, but update fading for a pair
; of tubes
editor_refresh_2
	call	editor_display_normal
	call	editor_display_2_values
	lfsr	FSR1,auto_brightness
	rlncf	ed_tube,w
	addwf	FSR1L,f
#if NUM_TUBES == 6
	incf	FSR1L,f
#endif
	movlw	(CYCLE_BRIGHT | EVERY_1_32_S)
	movwf	POSTINC1
	movwf	POSTINC1
	return

; 
; set the display : no fading, full brightness (ignore max_bright)
editor_display_normal
	bcf	INTCON,GIEL
	lfsr	FSR1,auto_brightness
	lfsr	FSR2,auto_fading
	movlw	NUM_TUBES
	movwf	ed_tmp
edn_0	movlw	(DIRECT_BRIGHT | 0xF)	; full brightness
	movwf	POSTINC1
	movlw	NO_FADING
	movwf	POSTINC2
	decfsz	ed_tmp,f
	bra	edn_0
#if SUPPORT_DP == 1
	lfsr	FSR1,auto_dp
	movlw	NUM_TUBES
	movwf	ed_tmp
edn_1	movlw	(DP_DIRECT | DP_OFF)
	movwf	POSTINC1
	decfsz	ed_tmp,f
	bra	edn_1
#endif
	bsf	INTCON,GIEL
	return	
	
	
;
; set the display to show editor values
;
SET_DIGIT MACRO
	iorlw	DIRECT_VALUE
	movwf	POSTINC1
	movwf	POSTINC1
	ENDM

;
; Display 4 ed_values on the tubes
; If the display has 6 tubes, the first and last tubes are turned off.
;
editor_display_4_values
	lfsr	FSR1,auto_values
	lfsr	FSR2,ed_values
#if NUM_TUBES == 6
	movlw	0xF
	call	ed4set
#endif
	movlw	.4
	movwf	ed_tmp
ed4_0	movf	POSTINC2,w
	call	ed4set
	decfsz	ed_tmp,f
	bra	ed4_0
#if NUM_TUBES == 6
	movlw	0xF
	call	ed4set
#endif
	return
ed4set	andlw	0x0F
	iorlw	DIRECT_VALUE
	movwf	POSTINC1
	movwf	POSTINC1
	return
		

;
; display 2 ed_values on the 4 tubes (ed_values in BCD)
;
editor_display_2_values
	lfsr	FSR1,auto_values
	lfsr	FSR2,ed_values
#if NUM_TUBES == 6
	movlw	0xF
	call	ed4set
#endif
	movlw	.2
	movwf	ed_tmp
ed2_0	swapf	INDF2,w
	call	ed4set
	movf	POSTINC2,w
	call	ed4set
	decfsz	ed_tmp,f
	bra	ed2_0
#if NUM_TUBES == 6
	movlw	0xF
	call	ed4set
#endif
	return
	

	END
	