;;;
;;; Copyright (C) 2010 Frederic Vecoven
;;;
;;; This file is part of NixieClock
;;;
;;; NixieClock is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 3 of the License, or
;;; (at your option) any later version.
;;;
;;; NixieClock is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program. If not, see <http://www.gnu.org/licenses/>.
;;;
	
;
; Program routines
;
	include hv.inc
	include program.inc
	
; variables
	extern	value, day_ptr, hour_ptr, minute_ptr, qsec_ptr
	extern	tube_mask, acount, time, tick, PFlag, CFlag, PrFlag
	extern	auto_values, auto_fading, auto_brightness, auto_flags
	extern	tcount, stack, stack_ptr
#if SUPPORT_DP == 1
	extern	auto_dp
#endif
	
; functions
	extern	set_tube, rand_W_16
	extern	power_off

; ROM data
	extern	day_prog
	
	
program_code	CODE

;
; See program.inc for an explanation of the program format.
;


;
; Given the current time, recompute the program pointers
;
init_ptrs
	GLOBAL	init_ptrs
	clrf	TBLPTRU				; initialize TBLPTR
	movlw	high(day_prog)
	movwf	day_ptr+1
	movlw	low(day_prog)
	movwf	day_ptr
	call	update_day_ptr
	call	update_hour_ptr
	call	update_minute_ptr
	call	update_qsec_ptr
	return
	
update_week_ptr
	GLOBAL	update_week_ptr
	movff	day_ptr+2,day_ptr		; restore day_ptr
	movff	day_ptr+3,day_ptr+1
	return
	
update_day_ptr
	GLOBAL	update_day_ptr
	movff	hour_ptr+2,hour_ptr		; restore hour_ptr
	movff	hour_ptr+3,hour_ptr+1
	movff	day_ptr+1,TBLPTRH
	movff	day_ptr,TBLPTRL
	movff	t_dow,value			; value = day of week
	call	update_ptr2
	btfsc	PrFlag, UPDATE_PTR_DAY
	return
	movff	TBLPTRH,day_ptr+1
	movff	TBLPTRL,day_ptr
	return
	
update_hour_ptr
	GLOBAL	update_hour_ptr
	movff	minute_ptr+2,minute_ptr		; restore minute_ptr
	movff	minute_ptr+3,minute_ptr+1
	movff	hour_ptr+1,TBLPTRH
	movff	hour_ptr,TBLPTRL
	movff	t_hour,value			; value = hours
	call	update_ptr2
	btfsc	PrFlag, UPDATE_PTR_HOUR
	return
	movff	TBLPTRH,hour_ptr+1
	movff	TBLPTRL,hour_ptr
	return
	
update_minute_ptr
	GLOBAL	update_minute_ptr
	movff	qsec_ptr+2,qsec_ptr		; restore qsec_ptr
	movff	qsec_ptr+3,qsec_ptr+1
	movff	minute_ptr+1,TBLPTRH
	movff	minute_ptr,TBLPTRL
	movff	t_min,value			; value = minutes
	call	update_ptr2
	btfsc	PrFlag, UPDATE_PTR_MIN
	return
	movff	TBLPTRH,minute_ptr+1
	movff	TBLPTRL,minute_ptr
	return
	
update_qsec_ptr
	GLOBAL	update_qsec_ptr
	movff	qsec_ptr+1,TBLPTRH
	movff	qsec_ptr,TBLPTRL
	movff	t_sec,value+1		; upper byte = seconds
	UNBCD	value+1
	movf	tick,w			; lower byte = 1/64th seconds
	andlw	0x3F
	movwf	value
	call	update_ptr
	btfsc	PrFlag, UPDATE_PTR_QSEC
	return
	movff	TBLPTRH,qsec_ptr+1
	movff	TBLPTRL,qsec_ptr
	return

;
; Read threshold from ROM, and execute the associated actions. Do this
; while the treshold is < or == to value.
;
; Input: TBLPTR, value
;
update_ptr2
	clrf	value+1			; used by day/hours/minutes
	UNBCD	value
update_ptr
	clrf	PrFlag
up_00
	TBLRD*				; TABLAT = threshold high
	incf	TABLAT,w
	btfsc	STATUS,Z		; if 0xff (END_PROGRAM), return
	return
	
	movf	value+1,w		; compare high byte
	cpfsgt	TABLAT
	bra	up_01
	return
up_01	TBLRD+*				; compare low byte
	movf	value,w
	cpfsgt	TABLAT			; if (threshold_L > value_L)
	bra	up_02
	TBLRD*-
	return				; then return
up_02
	TBLRD+*				; skip threshold
	call	exec_actions		; execute associated actions
	bra	up_00			; and loop

;
; Execute actions. TBLPTR points to the first action data
;
exec_actions
	TBLRD*+				; TABLAT = action (1st byte)
	movlw	END_ACTION_V
	cpfseq	TABLAT			; TABLAT == END_ACTION ?
	bra	ea_00			; no. 
	return				; yes, we are done
ea_00
	call	exec_one_action
	bra	exec_actions

;
; Execute one action. TBPTR points to first byte of action
;
exec_one_action
	movff	TABLAT,tube_mask+1	; save action + upper part of mask
	TBLRD*+
	movff	TABLAT,tube_mask	; save lower part of mask (or upper
					; byte of day/hour/min/qsec ptr)
	TBLRD*+				; read action data (or lower byte of
					; ptr)
	swapf	tube_mask+1,w		; get instruction (4 MSB)
	andlw	0x0F
	call	jump_table
	goto	act_set_value		; SET_VALUE
	goto	act_set_bright		; SET_BRIGHTNESS
	goto	act_set_fading		; SET_FADING
	goto	act_day_ptr		; SET_DAY_PTR
	goto	act_hour_ptr		; SET_HOUR_PTR
	goto	act_minute_ptr		; SET_MINUTE_PTR
	goto	act_qsec_ptr		; SET_QSEC_PTR
	goto	act_rotate		; SET_ROTATE
	goto	act_preset		; SET_PRESET
	goto	act_power		; SET_POWER
	goto	act_options		; SET_OPTIONS
	goto	act_set_dp		; SET_DP
	goto	act_gosub		; SUBROUTINE
	goto	act_none
	goto	act_none
	goto	act_none

;;;
;;; no action
;;; 
act_none
	return


;;;
;;; gosub subroutine
;;; 
act_gosub
	call	push_tblptr		; save TBLPTR in stack
	movff	tube_mask,TBLPTRH	; TBLPTR = address of subroutine
	movff	TABLAT,TBLPTRL
	call	exec_actions		; execute actions of subroutine
	call	pop_tblptr		; restore TBLPTR from stack
	return


	
;;;
;;; set day pointer
;;; 
act_day_ptr
	bsf	PrFlag,UPDATE_PTR_DAY
	call	random_select_ptr
	movff	tube_mask,day_ptr+1
	movff	tube_mask+2,day_ptr
	movff	day_ptr,day_ptr+2
	movff	day_ptr+1,day_ptr+3

	return

;;;
;;; set hour pointer
;;; 	
act_hour_ptr
	bsf	PrFlag,UPDATE_PTR_HOUR
	call	random_select_ptr
	movff	tube_mask,hour_ptr+1
	movff	tube_mask+2,hour_ptr
	movff	hour_ptr,hour_ptr+2
	movff	hour_ptr+1,hour_ptr+3
	return
	

;;;
;;; set minute pointer
;;; 	
act_minute_ptr
	bsf	PrFlag,UPDATE_PTR_MIN
	call	random_select_ptr
	movff	tube_mask,minute_ptr+1
	movff	tube_mask+2,minute_ptr
	movff	minute_ptr,minute_ptr+2
	movff	minute_ptr+1,minute_ptr+3
	return

	
;;;
;;; set quarter of second pointer
;;; 	
act_qsec_ptr
	bsf	PrFlag,UPDATE_PTR_QSEC
	call	random_select_ptr
	movff	tube_mask,qsec_ptr+1
	movff	tube_mask+2,qsec_ptr
	movff	qsec_ptr,qsec_ptr+2
	movff	qsec_ptr+1,qsec_ptr+3
	return
	

;;;
;;; set value
;;; 
act_set_value
	movf	tube_mask+1,w		; filter non-mask bits
	andlw	0x0F
	movwf	tube_mask+1
	clrf	acount			; acount = 0
asv_00
	rrcf	tube_mask+1,f		; get one bit from mask
	rrcf	tube_mask,f
	btfsc	STATUS,C
	call	asv_10			; set auto_value if bit == 1
	incf	acount,f		; acount++
	movlw	NUM_TUBES*2
	cpfseq	acount
	bra	asv_00			; loop until acount == NUM_TUBES*2
	return
asv_10
	lfsr	FSR1,auto_values
	movf	acount,w
	movff	TABLAT,PLUSW1		; auto_values[acount] = TABLAT
	return


;;;
;;; set brightness
;;; 
act_set_bright
	lfsr	FSR1,auto_brightness
	bra	copy_values

	
;;;
;;; set fading
;;;
act_set_fading
	lfsr	FSR1,auto_fading
	bra	copy_values

	
;;;
;;; set dot
;;;
act_set_dp
#if SUPPORT_DP == 1
	lfsr	FSR1,auto_dp
	bra	copy_values
#else
	return
#endif

	
; subroutine : copy TABLAT to un-masked tubes. (Skip alternate "tubes")
copy_values
	movf	tube_mask+1,w		; filter non-mask bits
	andlw	0x0F
	movwf	tube_mask+1
	clrf	acount			; acount = 0
copy_00
	rrcf	tube_mask+1,f		; get one bit from mask
	rrcf	tube_mask,f
	btfsc	STATUS,C
	call	copy_10			; set value if bit == 1
	rrcf	tube_mask+1,f		; skip alternate tube bit mask
	rrcf	tube_mask,f
	incf	acount,f		; acount++
	movlw	NUM_TUBES
	cpfseq	acount
	bra	copy_00			; loop until acount == NUM_TUBES
	return
copy_10
	movf	acount,w
	movff	TABLAT,PLUSW1		; FSR1[acount] = TABLAT
	return


;;;
;;; rotate
;;; 
act_rotate
	movf	TABLAT,w		; 0 = left, 1 = right
	bz	rotate_left
	bra	rotate_right

rotate_left
	btfss	tube_mask,R_FADING_BIT
	bra	rotl_00
	lfsr	FSR1,auto_fading
	call	rl_00
rotl_00
	btfss	tube_mask,R_BRIGHT_BIT
	bra	rotl_01
	lfsr	FSR1,auto_brightness
	call	rl_00
	lfsr	FSR1,auto_flags
	call	rl_00
rotl_01
	btfss	tube_mask,R_VALUE_BIT
	bra	rotl_02
	lfsr	FSR1,auto_values
	call	rl_01
rotl_02
	btfss	tube_mask,R_ALT_VALUE_BIT
	bra	rotl_03
	lfsr	FSR1,auto_values+1
	call	rl_01
rotl_03
#if SUPPORT_DP == 1
	btfss	tube_mask,R_DP_BIT
	bra	rotl_04
	lfsr	FSR1,auto_dp
	call	rl_00
rotl_04
#endif
	return
rl_00
	movff	POSTINC1,acount		; save (1)
	movlw	-2
	movff	POSTINC1,PLUSW1		; 2 -> 1
	movff	POSTINC1,PLUSW1		; 3 -> 2
	movff	POSTINC1,PLUSW1		; 4 -> 3
#if NUM_TUBES == 6
	movff	POSTINC1,PLUSW1		; 5 -> 4
	movff	POSTINC1,PLUSW1		; 6 -> 5
#endif
	movlw	-1
	movff	acount,PLUSW1		; (1) -> 4 or 6
	return
rl_01
	movff	POSTINC1,acount		; save (1)
	incf	FSR1L,f
	movlw	-3
	movff	POSTINC1,PLUSW1		; 2 -> 1
	incf	FSR1L,f
	movff	POSTINC1,PLUSW1		; 3 -> 2
	incf	FSR1L,f
	movff	POSTINC1,PLUSW1		; 4 -> 3
#if NUM_TUBES == 6
	incf	FSR1L,f
	movff	POSTINC1,PLUSW1		; 5 -> 4
	incf	FSR1L,f
	movff	POSTINC1,PLUSW1		; 6 -> 5
#endif
	movlw	-1
	movff	acount,PLUSW1		; (1) -> 4 or 6
	return
	
rotate_right
	btfss	tube_mask,R_FADING_BIT
	bra	rotr_00
	lfsr	FSR1,auto_fading+(NUM_TUBES-1)
	call	rr_00
rotr_00
	btfss	tube_mask,R_BRIGHT_BIT
	bra	rotr_01
	lfsr	FSR1,auto_brightness+(NUM_TUBES-1)
	call	rr_00
	lfsr	FSR1,auto_flags+(NUM_TUBES-1)
	call	rr_00
rotr_01
	btfss	tube_mask,R_VALUE_BIT
	bra	rotr_02
	lfsr	FSR1,auto_values+(2*NUM_TUBES-1)
	call	rr_01
rotr_02
	btfss	tube_mask,R_ALT_VALUE_BIT
	bra	rotr_03
	lfsr	FSR1,auto_values+(2*NUM_TUBES-2)
	call	rr_01
rotr_03
#if SUPPORT_DP == 1
	btfss	tube_mask,R_DP_BIT
	bra	rotr_04
	lfsr	FSR1,auto_dp+(NUM_TUBES-1)
	call	rr_00
rotr_04
#endif
	return
rr_00
	movff	POSTDEC1,acount		; save (4 or 6)
	movlw	2
#if NUM_TUBES == 6
	movff	POSTDEC1,PLUSW1		; 5 -> 6
	movff	POSTDEC1,PLUSW1		; 4 -> 5
#endif
	movff	POSTDEC1,PLUSW1		; 3 -> 4
	movff	POSTDEC1,PLUSW1		; 2 -> 3
	movff	POSTDEC1,PLUSW1		; 1 -> 2
	movlw	1
	movff	acount,PLUSW1		; (4 or 6) -> 1
	return
rr_01
	movff	POSTDEC1,acount		; save (4 or 6)
	decf	FSR1L,f
	movlw	3
#if NUM_TUBES == 6
	movff	POSTDEC1,PLUSW1		; 5 -> 6
	decf	FSR1L,f
	movff	POSTDEC1,PLUSW1		; 4 -> 5
	decf	FSR1L,f
#endif	
	movff	POSTDEC1,PLUSW1		; 3 -> 4
	decf	FSR1L,f
	movff	POSTDEC1,PLUSW1		; 2 -> 3
	decf	FSR1L,f
	movff	POSTDEC1,PLUSW1		; 1 -> 2
	movlw	1
	movff	acount,PLUSW1		; (4 or 6) -> 1
	return
	

;;;
;;; Directly set the value of a tube (i.e. when this action is executed)
;;; This is useful before auto incrementing or decrementing values, in order
;;; to set an initial value.
;;;
act_preset
	movf	tube_mask+1,w		; filter non-mask bits
	andlw	0x0F
	movwf	tube_mask+1
	clrf	tcount			; acount = 0
apr_00
	rrcf	tube_mask+1,f		; get one bit from mask
	rrcf	tube_mask,f
	btfsc	STATUS,C
	call	apr_10			; set direct value if bit == 1
	incf	tcount,f		; acount++
	movlw	NUM_TUBES*2
	cpfseq	tcount
	bra	apr_00			; loop until acount == NUM_TUBES*2
	return
apr_10
	movf	TABLAT,w
	call	set_tube
	return

	
;;;
;;; set power
;;;	
act_power
	movf	TABLAT,w		; 0 = off   1 = on
	bz	ap_to_off
	btfss	PFlag,POWER_USER_OFF	; don't power on if the user has forced
	bsf	PFlag,POWERING_ON	; the power to off.
	return
ap_to_off
	btfss	PFlag,POWERING_ON	; ignore the command if we are currently
	call	power_off		; powering on (power_on calls init_ptrs)
	return


;;;
;;; set options
;;;	
act_options
	btfss	tube_mask+1,0		; check if we should add or remove
	bra	options_remove
	movf	TABLAT,w		; get options
	iorwf	CFlag,f			; OR with current
	return
options_remove
	comf	TABLAT,w		; get !options
	andwf	CFlag,f			; AND with current
	return

	
;
; Given a list of pointers to [day,hour,minute,qsec] subroutines, pick
; one randomly and go there.
;
; This routine is called from act_[day,hour,minute,qsec]_ptr.
; Initially :
;   tube_mask+1 = action (4 bits) and upper mask
;   tube_mask   = high byte of pointer
;   TBLPTR      points to low byte of pointer
;
; The upper mask (4 bits) represents the total number of possible pointers
; minus one.
;
; The algorithm is simple :
; 1) save TBLPTR
; 2) TBLPTR -= 2     (TBLPTR points now to high byte of 1st pointer of the list
; 3) R = random (max random value = upper mask)
; 4) TBLPTR += 2 * R  (TBLPTR points now to the high byte of the chosen pointer)
; 5) read chosen pointer, and save into tube_mask (high) and tube_mask+2 (low)
; 6) restore TBLPTR
; 7) TBLPTR += 2 * upper mask (TBLPTR points to next action)
;
random_select_ptr
	call	push_tblptr		; save TBLPTR
	TBLRD*-				; TBLPTR -= 2
	TBLRD*-
	movf	tube_mask+1,w
	andlw	0x0F
	call	rand_W_16		; get a random number from [0, W]
	rlncf	WREG,w			; W *= 2
	addwf	TBLPTRL,f		; add W to TBLPTR
	btfsc	STATUS,C
	incf	TBLPTRH,f
	TBLRD*+				; read selected pointer
	movff	TABLAT,tube_mask
	TBLRD*+
	movff	TABLAT,tube_mask+2
	call	pop_tblptr		; restore TBLPTR
	movf	tube_mask+1,w		; get tube_mask
	andlw	0x0F			; mask bits
	rlncf	WREG,w			; multiply by 2
	addwf	TBLPTRL,f		; add to TBLPTR
	btfsc	STATUS,C
	incf	TBLPTRH,f
	return
	

;;; 
;;; push TBLPTR on stack
;;; 
push_tblptr
	lfsr	FSR1,stack
	movf	stack_ptr,w
	movff	TBLPTRH,PLUSW1
	incf	WREG,w
	movff	TBLPTRL,PLUSW1
	incf	WREG,w
	movwf	stack_ptr
	return


;;;
;;; pop TBLPTR from stack
;;; 
pop_tblptr
	lfsr	FSR1,stack
	decf	stack_ptr,w
	movff	PLUSW1,TBLPTRL
	decf	WREG,w
	movff	PLUSW1,TBLPTRH
	movwf	stack_ptr
	return

	
;;;
;;; jump table : modify return address by adding W*4 to it.
;;;
jump_table
	GLOBAL	jump_table
	rlncf	WREG,w
	rlncf	WREG,w
	addwf	TOSL,f
	movlw	0x00
	addwfc	TOSH,f
	;addwfc	TOSU,f			; if more than 64kB
	return
	
	END

