;;;
;;; Copyright (C) 2010 Frederic Vecoven
;;;
;;; This file is part of NixieClock
;;;
;;; NixieClock is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 3 of the License, or
;;; (at your option) any later version.
;;;
;;; NixieClock is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program. If not, see <http://www.gnu.org/licenses/>.
;;;
	
; 
; Program definitions
;

;
; For each day, hours, minutes and quarter-of-second, a program
; can specify how the tubes should be updated, by changing the
; auto_xxx variables (these are used to set values, fading and
; brightness of each tube)
;
; A program step is made of 
; - a value : when the following actions must be executed
; - a serie of actions, ended by END_ACTION
;
; A program is made of
; - one or more program steps
; - END_PROGRAM
;
; An action is defined by 3 bytes, where
; - the 4 MSB of the first byte define the action to do
; - the 4 LSB + the 2nd byte define the tube mask (up to 6 tubes). 
;   2 bits per tube, since we can set alternate values for fading.
; - the last byte is the value that will be stored in the relevant
;   ram for the auto_xxx routines.
;
; To facilitate the programming, macros are defined.
;
; Do NOT change the SET_* definitions below without changing the
; jump table (in program.asm) at the same time.
;

END_ACTION_V		EQU	0xFF

SET_VALUE		EQU	(0x0 << 4)
SET_BRIGHTNESS		EQU	(0x1 << 4)
SET_FADING		EQU	(0x2 << 4)
SET_DAY_PTR		EQU	(0x3 << 4)
SET_HOUR_PTR		EQU	(0x4 << 4)
SET_MINUTE_PTR		EQU	(0x5 << 4)
SET_QSEC_PTR		EQU	(0x6 << 4)
SET_ROTATE		EQU	(0x7 << 4)
SET_PRESET		EQU	(0x8 << 4)
SET_POWER		EQU	(0x9 << 4)
SET_OPTIONS		EQU	(0xA << 4)
SET_DP			EQU	(0xB << 4)
SUBROUTINE		EQU	(0xC << 4)

; day of week (see time.asm 'get_dow')
SUNDAY			EQU	0x0
MONDAY			EQU	0x1
TUESDAY			EQU	0x2
WEDNESDAY		EQU	0x3
THURSDAY		EQU	0x4
FRIDAY			EQU	0x5
SATURDAY		EQU	0x6

; tube masks (there is room for 6 tubes)
TUBE_1			EQU	0x001
TUBE_1_ALT		EQU	0x002
TUBE_2			EQU	0x004
TUBE_2_ALT		EQU	0x008
TUBE_3			EQU	0x010
TUBE_3_ALT		EQU	0x020
TUBE_4			EQU	0x040
TUBE_4_ALT		EQU	0x080
TUBE_5			EQU	0x100
TUBE_5_ALT		EQU	0x200
TUBE_6			EQU	0x400
TUBE_6_ALT		EQU	0x800
TUBE_11			EQU	(TUBE_1 | TUBE_1_ALT)
TUBE_22			EQU	(TUBE_2 | TUBE_2_ALT)
TUBE_33			EQU	(TUBE_3 | TUBE_3_ALT)
TUBE_44			EQU	(TUBE_4 | TUBE_4_ALT)
TUBE_55			EQU	(TUBE_5 | TUBE_5_ALT)
TUBE_66			EQU	(TUBE_6 | TUBE_6_ALT)
TUBE_1234		EQU	(TUBE_1 | TUBE_2 | TUBE_3 | TUBE_4)
TUBE_1234_ALT		EQU	(TUBE_1_ALT | TUBE_2_ALT | TUBE_3_ALT | TUBE_4_ALT)
TUBE_123456		EQU	(TUBE_1234 | TUBE_5 | TUBE_6)
TUBE_123456_ALT		EQU	(TUBE_1234_ALT | TUBE_5_ALT | TUBE_6_ALT)
TUBE_ALL		EQU	(TUBE_123456 | TUBE_123456_ALT)

	
; rotate masks
R_VALUE_BIT		EQU	0
R_ALT_VALUE_BIT		EQU	1
R_FADING_BIT		EQU	2
R_BRIGHT_BIT		EQU	3
R_DP_BIT		EQU	4
R_VALUE			EQU	(1 << R_VALUE_BIT)
R_ALT_VALUE		EQU	(1 << R_ALT_VALUE_BIT)
R_FADING		EQU	(1 << R_FADING_BIT)
R_BRIGHT		EQU	(1 << R_BRIGHT_BIT)
R_DP			EQU	(1 << R_DP_BIT)
R_ALL_NO_DP		EQU	(R_VALUE | R_ALT_VALUE | R_FADING | R_BRIGHT)
R_ALL			EQU	(R_ALL_NO_DP | R_DP)

; auto-set values (see clock.asm, update_values)
NO_CHANGE_VALUE		EQU	(0x0 << 4)
INC_VALUE		EQU	(0x1 << 4)
DEC_VALUE		EQU	(0x2 << 4)
RANDOM_VALUE		EQU	(0x3 << 4)
DIRECT_VALUE		EQU	(0x4 << 4)
AUTO_VALUE		EQU	(0x1 << 7)

; data from 'time'
SECONDS_H		EQU	((0x00 << 1) | 1)
SECONDS_L		EQU	((0x00 << 1) | 0)
MINUTES_H		EQU	((0x01 << 1) | 1)
MINUTES_L		EQU	((0x01 << 1) | 0)
HOURS24_H		EQU	((0x02 << 1) | 1)
HOURS24_L		EQU	((0x02 << 1) | 0)
DAY_OF_WEEK_H		EQU	((0x03 << 1) | 1)
DAY_OF_WEEK_L		EQU	((0x03 << 1) | 0)
DAY_H			EQU	((0x04 << 1) | 1)
DAY_L			EQU	((0x04 << 1) | 0)
MONTH_H			EQU	((0x05 << 1) | 1)
MONTH_L			EQU	((0x05 << 1) | 0)
YEAR_H			EQU	((0x06 << 1) | 1)
YEAR_L			EQU	((0x06 << 1) | 0)

; data from 'time'+1sec
SECONDS_1_H		EQU	((0x07 << 1) | 1)
SECONDS_1_L		EQU	((0x07 << 1) | 0)
MINUTES_1_H		EQU	((0x08 << 1) | 1)
MINUTES_1_L		EQU	((0x08 << 1) | 0)
HOURS24_1_H		EQU	((0x09 << 1) | 1)
HOURS24_1_L		EQU	((0x09 << 1) | 0)
DAY_OF_WEEK_1_H		EQU	((0x0A << 1) | 1)
DAY_OF_WEEK_1_L		EQU	((0x0A << 1) | 0)
DAY_1_H			EQU	((0x0B << 1) | 1)
DAY_1_L			EQU	((0x0B << 1) | 0)
MONTH_1_H		EQU	((0x0C << 1) | 1)
MONTH_1_L		EQU	((0x0C << 1) | 0)
YEAR_1_H		EQU	((0x0D << 1) | 1)
YEAR_1_L		EQU	((0x0D << 1) | 0)

; offset between current time and time + 1 sec
TIME1_OFFSET		EQU	(SECONDS_1_L - SECONDS_L)

; day or month (option bit to select DD/MM or MM/DD easily)
DATE_PART1_MASK		EQU	0x20
DATE_PART1_H		EQU	((DATE_PART1_MASK << 1) | 1)
DATE_PART1_L		EQU	((DATE_PART1_MASK << 1) | 0)	
DATE_PART2_MASK		EQU	0x21
DATE_PART2_H		EQU	((DATE_PART2_MASK << 1) | 1)
DATE_PART2_L		EQU	((DATE_PART2_MASK << 1) | 0)	
DATE_PART1_1_MASK	EQU	0x22
DATE_PART1_1_H		EQU	((DATE_PART1_1_MASK << 1) | 1)
DATE_PART1_1_L		EQU	((DATE_PART1_1_MASK << 1) | 0)	
DATE_PART2_1_MASK	EQU	0x23
DATE_PART2_1_H		EQU	((DATE_PART2_1_MASK << 1) | 1)
DATE_PART2_1_L		EQU	((DATE_PART2_1_MASK << 1) | 0)	
	
; hours (format depends on option bits)
HOURS_MASK		EQU	0x24
HOURS_H			EQU	((HOURS_MASK << 1) | 1)
HOURS_L			EQU	((HOURS_MASK << 1) | 0)
HOURS_1_MASK		EQU	0x25
HOURS_1_H		EQU	((HOURS_1_MASK << 1) | 1)
HOURS_1_L		EQU	((HOURS_1_MASK << 1) | 0)

; frequency def
EVERY_2S		EQU	0x0
EVERY_1S		EQU	0x1
EVERY_1_2_S		EQU	0x2
EVERY_1_4_S		EQU	0x3
EVERY_1_8_S		EQU	0x4
EVERY_1_16_S		EQU	0x5
EVERY_1_32_S		EQU	0x6
EVERY_1_64_S		EQU	0x7

; brightness defs (see clock.asm, update_brightness)
NO_CHANGE_BRIGHT	EQU	(0x0 << 5)
INC_BRIGHT		EQU	(0x1 << 5)
DEC_BRIGHT		EQU	(0x2 << 5)
CYCLE_BRIGHT		EQU	(0x3 << 5)
DIRECT_BRIGHT		EQU	(0x4 << 5)

; fading defs (see clock.asm, update_fading)
NO_FADING		EQU	(0x0 << 5)
FADING_TYPE_1		EQU	(0x1 << 5)
FADING_TYPE_2		EQU	(0x2 << 5)
FADING_FIXED		EQU	(0x7 << 5)

; dp defs (see clock.asm, update_dp)
NO_CHANGE_DP		EQU	(0x0 << 6)
DP_TYPE_1		EQU	(0x1 << 6)
DP_DIRECT		EQU	(0x3 << 6)
DP_OFF			EQU	0x0
DP_ON			EQU	0x1
DP_TOGGLE		EQU	0x2

; clock options (see clock.asm, value_autoset)
OPT_MODE12		EQU	(1 << MODE12_BIT)
OPT_ZERO_SUPPRESS	EQU	(1 << ZERO_SUPPRESS_BIT)
OPT_MMDD		EQU	(1 << DATE_MMDD_BIT)


; defs for macro parameters
DIRECT		EQU	0x00
INCREMENT	EQU	0x01
DECREMENT	EQU	0x02
AUTO		EQU	0x03
CYCLE		EQU	0x04
RIGHT		EQU	0x05
LEFT		EQU	0x06
NO_CHANGE	EQU	0x07
RANDOM		EQU	0x08
FADING		EQU	0x09
REPEAT		EQU	0x0A
TYPE		EQU	0x0B


;
; MACRO
;
DAY_PTR	MACRO label
	db	SET_DAY_PTR, high(label), low(label)
	ENDM
	
DAY_RAND_PTR	MACRO n
	db	(SET_DAY_PTR | ((n - 1) & 0x0F))
	ENDM
	
HOUR_PTR MACRO label
	db	SET_HOUR_PTR, high(label), low(label)
	ENDM

HOUR_RAND_PTR	MACRO n
	db	(SET_HOUR_PTR | ((n - 1) & 0x0F))
	ENDM
		
MINUTE_PTR MACRO label
	db	SET_MINUTE_PTR, high(label), low(label)
	ENDM

MINUTE_RAND_PTR	MACRO n
	db	(SET_MINUTE_PTR | ((n - 1) & 0x0F))
	ENDM
	
QSEC_PTR MACRO label
	db	SET_QSEC_PTR, high(label), low(label)
	ENDM	
	
QSEC_RAND_PTR	MACRO n
	db	(SET_QSEC_PTR | ((n - 1) & 0x0F))
	ENDM

PROG MACRO label
	db high(label), low(label)
	ENDM
	
GOSUB MACRO label
	db	SUBROUTINE, high(label), low(label)
	ENDM
		
TUBE_VALUE MACRO tube, action, x
	db	(SET_VALUE | (((tube) & 0xF00) >> 8))
	db	((tube) & 0xFF)
	if action == DIRECT
		db	DIRECT_VALUE | ((x) & 0x0F)
	else 
	if action == AUTO
		db	AUTO_VALUE | ((x) & 0x7F)
	else 
	if action == INCREMENT
		db	INC_VALUE | ((x) & 0x0F)
	else 
	if action == DECREMENT
		db	DEC_VALUE | ((x) & 0x0F)
	else
	if action == RANDOM
		db	RANDOM_VALUE | ((x) & 0x0F)
	else
	if action == NO_CHANGE
		db	NO_CHANGE_VALUE
	else
		error "Invalid action in macro TUBE_VALUE"
	endif
	endif
	endif
	endif
	endif
	endif
	ENDM

;
; Directly set the value of a tube. Contrary to TUBE_VALUE, the value
; is changed directly, while TUBE_VALUE schedules the change. This is
; not useful except when you want to use 'increment' or 'decrement'
; afterwards, so you know what value will be used initially. (You can also
; know this by looking at your program, but that's a different story).
;
TUBE_PRESET MACRO tube, x
	db	(SET_PRESET | (((tube) & 0xF00) >> 8))
	db	((tube) & 0xFF)
	db	((x) & 0xF)
	ENDM
	
TUBE_BRIGHT MACRO tube, action, x
	db	(SET_BRIGHTNESS | (((tube) & 0xF00) >> 8))
	db	((tube) & 0xFF)
	if action == DIRECT
		db	DIRECT_BRIGHT | ((x) & 0x0F)
	else
	if action == INCREMENT
		db	INC_BRIGHT | ((x) & 0x1F)
	else
	if action == DECREMENT
		db	DEC_BRIGHT | ((x) & 0x1F)
	else 
	if action == CYCLE
		db	CYCLE_BRIGHT | ((x) & 0x1F)
	else
	if action == NO_CHANGE
		db	NO_CHANGE_BRIGHT
	else
		error "Invalid action in macro TUBE_BRIGHT"
	endif
	endif
	endif
	endif
	endif
	ENDM
		
TUBE_FADING MACRO tube, action, x
	db	(SET_FADING | (((tube) & 0xF00) >> 8))
	db	((tube) & 0xFF)
	if action == DIRECT
		db	FADING_FIXED | ((x) & 0x1F)
	else
	if action == FADING
		db	FADING_TYPE_1 | ((x) & 0xF)
	else
	if action == REPEAT
		db	FADING_TYPE_2 | ((x) & 0x3)
	else
	if action == TYPE
		db	(x)
	else
		error "Invalid action in macro TUBE_FADING"
	endif
	endif
	endif
	endif
	ENDM

TUBE_DP MACRO tube, action, x
#if SUPPORT_DP == 1
	db	(SET_DP | (((tube) & 0xF00) >> 8))
	db	((tube) & 0xFF)
	if action == DIRECT
		db	DP_DIRECT | ((x) & 0x3)
	else
	if action == NO_CHANGE
		db	NO_CHANGE_DP
	endif
	endif
#endif
	ENDM

TUBE_DP_T1 MACRO tube, delay, duration
#if SUPPORT_DP == 1
	db	(SET_DP | (((tube) & 0xF00) >> 8))
	db	((tube) & 0xFF)
	db	DP_TYPE_1 | (((duration) & 0x7) << 3) | ((delay) & 0x7)
#endif
	ENDM

SET_POWER_ON MACRO
	db	SET_POWER, 0x00, 0x01
	ENDM

SET_POWER_OFF MACRO
	db	SET_POWER, 0x00, 0x00
	ENDM

SET_CLOCK_OPTIONS MACRO x
	db	SET_OPTIONS, 0x00, x
	ENDM

RESET_CLOCK_OPTIONS MACRO x
	db	SET_OPTIONS | 0x1, 0x00, x
	ENDM


	
ROTATE	MACRO	how
	db	SET_ROTATE, R_ALL
	if how == LEFT
		db	0x00
	else
	if how == RIGHT
		db	0x01
	else
		error "Invalid mode in macro ROTATE"
	endif
	endif
	ENDM
	
ROTATE2	MACRO	how, what
	db	SET_ROTATE, (what)
	if how == LEFT
		db	0x00
	else
	if how == RIGHT
		db	0x01
	else
		error "Invalid mode in macro ROTATE2"
	endif
	endif
	ENDM
	
	
ACTION MACRO x
	db	0x00, (x)
	ENDM
	
; specify in 1/64th sec increments
ACTION_QSEC MACRO x
	db	((x) / .64), ((x) % .64)
	ENDM
	
; specify in seconds and 1/64th.
ACTION_QSEC2 MACRO x, y
	db	(x), ((y) & 0x3F)
	ENDM

END_ACTION MACRO
	db	END_ACTION_V
	ENDM

END_SUB MACRO
	db	END_ACTION_V
	ENDM

END_PROGRAM MACRO
	db	0xff
	ENDM