;;;
;;; Copyright (C) 2010 Frederic Vecoven
;;;
;;; This file is part of NixieClock
;;;
;;; NixieClock is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 3 of the License, or
;;; (at your option) any later version.
;;;
;;; NixieClock is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program. If not, see <http://www.gnu.org/licenses/>.
;;;
	
;
; Various tables
;
	include hv.inc

	
	
tables_code	CODE


;
; value table : modify return address by adding W*2 to it
;
value_table
	GLOBAL	value_table
	rlncf	WREG,w
	addwf	TOSL,f
	movlw	0x00
	addwfc	TOSH,f
	;addwfc	TOSU,f			; if more than 64kB
	return


; digit mapping depends on your PCB. if the connections are directly
; made to the 74141, no mapping is necessary.

#if CLOCK_MODEL == 3
remap_digit
	GLOBAL	remap_digit
	call	value_table
	dt	0xc, 0x4, 0x0, 0x8, 0x1, 0xb, 0xa, 0x2, 0x3, 0x9
	dt	0xf, 0xf, 0xf, 0xf, 0xf, 0xf
	
unmap_digit
	GLOBAL	unmap_digit
	call	value_table
	dt	0x2, 0x4, 0x7, 0x8, 0x1, 0xf, 0xf, 0xf, 0x3, 0x9
	dt	0x6, 0x5, 0x0, 0xf, 0xf, 0xf
#endif


;
; - tubes 1-6 information is stored in pair0, pair1 and pair2
; - each pair is 8 bytes
; - in a pair, digit 1 refers to the rightmost digit on the display
;
;         fade1 bright1 fade2 bright2 digit1 digit2 digit1_alt digit2_alt
; pair0   0     1       2     3       4      5      6          7
; pair1   8     9       10    11      12     13     14         15
; pair2   16    17      18    19      20     21     22         23
; 
	
; tube number -> offset in 'pair' to its fading setting
remap_fading
	GLOBAL	remap_fading
#if NUM_TUBES == 4
	andlw	0x3
	call	value_table
	dt	.2, .10, .0, .8
#else
	andlw	0x7
	call	value_table
	dt	.2, .10, .18, .0, .8, .16
	dt	.0, .0			; N/A
#endif
	
; tube number -> offset in 'pair' to its brightness setting
;
; just use remap_fading, then add '1'.
	
	
; tube number -> offset in 'pair' to its digit value
remap_tube
	GLOBAL	remap_tube
#if NUM_TUBES == 4
	andlw	0x7
	call	value_table
	dt	.5, .7, .13, .15, .4, .6, .12, .14
#else
	andlw	0xF
	call	value_table
	dt	.5, .7, .13, .15, .21, .23
	dt	.4, .6, .12, .14, .20, .22
	dt	.0, .0, .0, .0		; N/A
#endif
	
; frequency setting -> mask for 'ticks'
freq_mask
	GLOBAL	freq_mask
	andlw	0x7
	call	value_table
	dt	0x7F, 0x3F, 0x1F, 0x0F, 0x07, 0x03, 0x1, 0x0
	
; number of days per month (in BCD, january offset = 1)
; W input in binary or BCD
days_per_month
	GLOBAL	days_per_month
	call	value_table
	dt	0x00
	dt	0x31, 0x28, 0x31, 0x30, 0x31, 0x30, 0x31, 0x31, 0x30
	dt	0x31, 0x30, 0x31
	dt	0x00, 0x00, 0x00
	dt	0x31, 0x30, 0x31

; months table [calculation of day of week]
; (January offset = 1)
; W input in binary or BCD
dow_months_table
	GLOBAL	dow_months_table
	call	value_table
	dt	0x0
	dt	0x0, 0x3, 0x3, 0x6, 0x1, 0x4, 0x6, 0x2, 0x5
	dt	0x0, 0x3, 0x5
	dt	0x0, 0x0, 0x0
	dt	0x0, 0x3, 0x5
	
; same as above, but for leap years (jan and feb are different)
dow_months_table_leap
	GLOBAL	dow_months_table_leap
	call	value_table
	dt	0x0
	dt	0x6, 0x2, 0x3, 0x6, 0x1, 0x4, 0x6, 0x2, 0x5
	dt	0x0, 0x3, 0x5
	dt	0x0, 0x0, 0x0
	dt	0x0, 0x3, 0x5
	

; table used in fading routines (precomputed divisions)
fading_table
	GLOBAL	fading_table
	call	value_table
	dt	0xff, 0x80, 0x55, 0x40, 0x33, 0x2a, 0x24, 0x20
	dt	0x1c, 0x19, 0x17, 0x15, 0x13, 0x12, 0x11, 0x10

	
	END
	